package main
import (
	"crypto/md5"; "encoding/hex"
	"fmt"; "log"; "flag"
	"runtime"; "time"
)

var hash [16]byte
var branch int

func init() {
	flag.IntVar(&branch, "b", -1, "number of parallel branches")
	multip := flag.Bool("m", false, "multiproc evaluation")
	flag.Parse()
	if branch > 0 { return }
	if !*multip {
		branch = 1
	} else {
		if branch < 0 { branch = runtime.GOMAXPROCS(-1) }
	}
}

func main() {
	hashString := "95ebc3c7b3b9f1d2c40fec14415d3cb8" // "zzzzz"
	if len(flag.Args()) > 0 {
		hashString = flag.Arg(0) 
	} else { 
		fmt.Scanf("%s", &hashString)
	}
	h, err := hex.DecodeString(hashString)
	if err != nil {
		log.Fatal(err)
	}
	copy(hash[0:len(hash)], h)
	fmt.Printf("число процессоров в системе: %v\n", runtime.NumCPU())
	fmt.Printf("число ветвей вычисления: %v\n", branch)
	fmt.Printf("хэш-строка: %v\n", hashString)

	in, out := make(chan string), make(chan string)
	t := time.Now()
	for i := 0; i < branch; i++ {
		go worker(in, out)
	}
	go generator(in)
	fmt.Println("пароль:", <-out)
	fmt.Println("время поиска:", time.Since(t))
}

// следующий символ словаря
var nextByte = func (b byte) byte {
	if 'z' == b {return '0'}
	if '9' == b {return 'a'}
	return b + 1
}

// worker по порядку сравнивает хэш каждой строки пароля с искомым
func worker(in <-chan string, out chan<- string) {
	var p string
	var b []byte
	for {
		p = <-in
		b = []byte(p)
		e := b[0]
		for b[0] == e {
			if md5.Sum(b) == hash {
				out <- string(b)                // хэш совпал - успех!
				return
			}
			for i := len(b) - 1; i >= 0; i-- { // следующая строка перебора
				b[i] = nextByte(b[i])
				if b[i] != '0' {
					break // return
				}
			}
		}
	}
}

// generator создаёт начальную строку группы перебора вида "X...."
func generator(in chan<- string) {
	start := []byte("00000")
	var b byte
	for {
		b = nextByte(start[0])
		in <- string(start)
		start[0] = b
		if b == '0' { return } // все возможные комбинации перебраны
	}
}
