package main

import (
	"errors"
	"fmt"
	"io"
	"math"
	"math/cmplx"
	"os"
	"strconv"
	"strings"
)

// -------------- класс точки вершины ---------------------
type point struct {
	xy complex128
}

func (p *point) String() string { // формат вывода
	return fmt.Sprintf("[%.2f,%.2f] ", real(p.xy), imag(p.xy))
}
func (p *point) inpoint() (ok bool, err error) { // ввод координат
	buf := make([]byte, 1024)
	ok = false
	n, err := os.Stdin.Read(buf)
	if err == io.EOF || n == 0 || buf[n-1] != '\n' { // конец ввода
		err = io.EOF
		return
	}
	as := strings.Split(string(buf[:n-1]), string(" "))
	if len(as) != 2 {
		err = errors.New("число параметров")
		return
	}
	x, err := strconv.ParseFloat(as[0], 64)
	if err != nil {
		return
	} // ошибка преобразования
	y, err := strconv.ParseFloat(as[1], 64)
	if err != nil {
		return
	} // ошибка преобразования
	p.xy = complex(x, y)
	ok, err = true, nil
	return
}

// ------------- класс многоугольника ---------------------
type shape []point

func (p *shape) append(data point) {
	slice := *p
	l := len(slice)
	if l+1 > cap(slice) { // недостаточно места
		newSlice := make([]point, (l+1)*2) // выделение вдвое большего буфера
		if l > 0 {                         // скопировать данные
			for i, c := range slice {
				newSlice[i] = c
			}
		}
		slice = newSlice
	}
	slice = slice[0 : l+1]
	slice[l] = data
	*p = slice
}
func (p *shape) String() string { // формат вывода
	slice := *p
	var s string = ""
	for _, c := range slice {
		s += c.String()
	}
	return s
}
func (p *shape) perimeter() float64 {
	summa := 0.0
	slice := *p
	for i, c := range slice {
		if i == 0 {
			summa += cmplx.Abs(c.xy - slice[len(slice)-1].xy)
		} else {
			summa += cmplx.Abs(c.xy - slice[i-1].xy)
		}
	}
	return summa
}
func (p *shape) square() float64 {
	summa := 0.0
	slice := *p
	for i := 0; i < len(slice)-2; i++ {
		r1, θ1 := cmplx.Polar(slice[i+1].xy - slice[0].xy)
		r2, θ2 := cmplx.Polar(slice[i+2].xy - slice[0].xy)
		summa += r1 * r2 * math.Abs(math.Sin(θ2-θ1)) / 2.
	}
	return summa
}

func main() {
	for {
		fmt.Println("координаты вершин в формате: X Y (^D конец ввода)")
		многоугольник := new(shape)
		i := 0
		точка := new(point)
		for {
			fmt.Printf("вершина № %v: ", i+1)
			ok, err := точка.inpoint() // ввод координат вершины
			if !ok {
				if err == io.EOF {
					fmt.Printf("\r")
					break
				} // конец ввода вершин
				fmt.Printf("ошибка ввода: %s!\n", err)
				continue
			}
			многоугольник.append(*точка)
			i++
		}
		fmt.Printf("вершин %d : %v\n", len(*многоугольник), многоугольник)
		fmt.Printf("периметр = %.2f\n", многоугольник.perimeter())
		fmt.Printf("площадь = %.2f\n", многоугольник.square())
		fmt.Println("---------------------------------")
	}
}

/*
http://archive.miek.nl/files/go/Learning-Go-latest.pdf
http://golang.org/src/pkg/
*/
