package main

//#include <pthread.h>
//unsigned long tid(void) { return (unsigned long)pthread_self(); }
import "C"
import (
	"fmt"; "os"; "runtime"
	"math"; "math/cmplx"
	"strconv"; "time"
)

type msg struct { // структура сообщения
	n int          // номер ветви
	t int64        // системный pthread_t
	d int          // задержка в ms
	s time.Time    // время старта
}
func (p msg) String() string {       // метод форматирования сообщения
	return fmt.Sprintf("[%02d,%X,%03d]", p.n, p.t, p.d)
}

func main() {
	fmt.Printf("число процессоров в системе: %v\n", runtime.NumCPU())
	ветви := 3
	debug := true
	if len(os.Args) > 1 {
		ветви, _ = strconv.Atoi(os.Args[1])
		if ветви < 0 {
			debug = false
			ветви = -ветви
		}
	}
	fmt.Printf("число ветвей выполнения: %v\n", ветви)

	активность := func(n int64) {     // функция активной паузы 
		var β, ω, t complex128 = 0.0, math.Pi / 5, 0.0
		const step = 1000
		n *= step
		for n != 0 {
			β += cmplx.Exp(-1i * ω * t)
			t += 0.1
			n--
		}
	}

	count := func() int64 {
		t0, t1 := time.Now(), time.Now()
		var count_1sec int64 = 0
		for t1.Sub(t0).Seconds() < 1 { // калибровка паузы 1s
			активность(1)
			count_1sec++
			t1 = time.Now()
		}
		return count_1sec
	}()

	delay := func() { активность(count) }

	t0 := time.Now()
	delay()
	if debug {
		fmt.Printf("%s\n", msg{0, int64(C.tid()), 
		           int(time.Now().Sub(t0).Milliseconds()), t0}.String())
	}
	var table = map[int64] int {int64(C.tid()) : 1}

	ch := make(chan msg)
	t0 = time.Now()
	for i := 1; i <= ветви; i++ {     // параллельные ветки
		t1 := time.Now()
		go func(n int) {
			delay()
			ch <- msg{int(n), int64(C.tid()), 0, t1}
		}(i)
	}
	for i := 1; i <= ветви; i++ {
		m := <-ch
		if debug { 
			m.d = int(time.Now().Sub(m.s).Milliseconds())
			fmt.Printf("%s\n", m.String())
		}
		elem, ok := table[m.t]
		if ok { 
			table[m.t] = elem + 1
		} else { table[m.t] = 1 }
	}
	fmt.Printf("итоговое время параллельного выполнения: %v\n",
	           time.Now().Sub(t0))
	for key, value := range table {   // гистограмма использования потоков
		fmt.Printf("%X => [%d]\n", key, value)
	}
}
