package main

import (
	"flag"
	"fmt"
	"math"
	"runtime"
	"sync"
	"time"
)

func main() {
	var branch int
	flag.IntVar(&branch, "b", 30, "number of parallel branches")
	useDebug := flag.Bool("v", false, "increase verbosity")
	flag.Parse()

	activDo := func(n int64) { // активная рабочая функция
		var β, θ float64 = 0.0, math.Pi / 2
		for n != 0 {
			β += math.Sin(θ)
			θ *= 1.1
			n--
		}
	}
	const norma = 100000000
	start := time.Now()
	activDo(int64(norma)) // калибровка
	var dt time.Duration
	dt = time.Since(start)

	множитель := float64(time.Second) / float64(dt) * norma
	индекс := int64(math.Round(множитель))
	if *useDebug {
		fmt.Printf("вызов калибровки: %dns | %f s\n", dt, dt.Seconds())
		fmt.Printf("множитель=%f\n", множитель)
		fmt.Printf("индекс для 1s = %d\n", индекс)
	}

	start = time.Now()
	activDo(индекс)       // единичное выполнение
	dt = time.Since(start)
	fmt.Printf("единичное выполнение:  %dns | %f s\n", dt, dt.Seconds())

	fmt.Printf("число процессоров в системе: %v\n", runtime.NumCPU())
	var wg sync.WaitGroup
	fmt.Printf("Выполнение %d ветвей...\n", branch)
	start = time.Now()
	for branch > 0 {
		wg.Add(1)          // увеличиваем WaitGroup счетчик
		go func() {
			defer wg.Done() // уменьшаем счетчик когда goroutine завершается
			if *useDebug {
				fmt.Println("Старт ", branch, time.Now())
			}
			activDo(индекс)
			if *useDebug {
				fmt.Println("Финиш ", branch, time.Now())
			}
		}()
		branch--
	}
	wg.Wait()             // ожидаем завершения всех goroutine
	dt = time.Since(start)
	fmt.Printf("... потребовало: %dns | %fs\n", dt, dt.Seconds())
}
